<?php
/*
 * Plugin Name: Codetutor Alipay for WooCommerce 
 * Plugin URI: http://www.victoriawy.com
 * Description: WooCommerce中进行支付宝支付
 * Author: Codetutor
 * Version: 0.1.0
 * Author URI:  http://www.victoriawy.com
 */

if (! defined ( 'ABSPATH' ))
    exit (); // Exit if accessed directly


require_once("alipay-sdk-PHP-20171121170331/AopSdk.php");


define( 'WC_Alipay_URL', plugins_url('', __FILE__) );     // 当前插件目录的URI, http://www.xxx.com/wp-content/plugins/woocommerce-alipay

/*
if(!class_exists('WC_Payment_Gateway')){
    return;
}
*/
// 在插件加载后应该创建一个类
add_action('plugins_loaded', 'init_my_gateway_class');
function init_my_gateway_class() {
    class WC_Gateway_Alipay extends WC_Payment_Gateway {

        public function __construct() {
            $this->id = "codetutor-alipay-for-wc";
            $this->icon = WC_Alipay_URL . "/assets/imgs/alipay.png";
            $this->has_fields = false;   
            $this->method_title = "支付宝";
            $this->method_description = "用支付宝进行结算";

            $this->init_form_fields();
            $this->init_settings();

            $this->enabled = $this->get_option("enabled");
            $this->title = $this->get_option("title");
            $this->description = $this->get_option("description");
            $this->appid = $this->get_option("appid");
            $this->appsecret = $this->get_option("appsecret");
            $this->signtype = $this->get_option("signtype");
            $this->alipaysecret = $this->get_option("alipaysecret");
            $this->pid = $this->get_option("pid");
            global $aop; 
            $aop = new AopClient();
            $aop->gatewayUrl = "https://openapi.alipay.com/gateway.do";
            //$aop->gatewayUrl = "https://openapi.alipaydev.com/gateway.do";
            $aop->appId = $this->appid;
            $aop->rsaPrivateKey = $this->appsecret; #应用私钥
            $aop->apiVersion = '1.0';
            $aop->signType = $this->signtype;
            $aop->alipayrsaPublicKey = $this->alipaysecret;
            $aop->postCharset = "utf-8";
            $aop->format = "json";
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        }

        function init_form_fields() {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'woocommerce'),
                    'type' => 'checkbox',
                    'lable' => __('Enable Alipay Payment', 'woocommerce'),
                    'default' => 'no'
                ),
                'title' => array(
                    'title' => __('Title', 'woocommerce'),
                    'type' => 'text',
                    'description' => __('用户在结算时会看到的标题', 'woocommerce'),
                    'default' => __('支付宝支付', 'woocommerce'),
                    'desc_tip' => true,
                    'css' => 'width:400px'
                ),

                'description' => array(
                    'title' => __('Alipay description', 'woocommerce'),
                    'type' => 'textarea',
                    'default' => '',
                    'css' => 'width:400px'
                ),
                'appid' => array(
                    'title' => __('APP ID', 'woocommerce'),
                    'type' => 'text',
                    'default' => '',
                    'description' => __('在支付宝开放平台上面创建的应用的APP ID，创建过程请阅读：<a href="https://docs.open.alipay.com/200/105310/" target="_blank">创建应用</a>', 'woocommerce'),
                    'css' => 'width:400px'
                ),

                'appsecret' => array(
                    'title' => __('APP私钥', 'woocommerce'),
                    'type' => 'text',
                    'default' => '',
                    'description' => __('在支付宝开放平台上面创建应用的时候生成的应用秘钥', 'woocommerce'),
                    'css' => 'width:400px'
                ),
                'signtype' => array(
                    'title' => __('加签方式', 'woocommerce'),
                    'type' => 'text',
                    'default' => '',
                    'description' => __('在支付宝开放平台上面创建应用的时候设置的加签方式，若使用SHA256WithRSA签名算法则sign type=RSA2，如果是SHA1WithRSA，那么sign type=RSA', 'woocommerce'),
                    'css' => 'width:400px'
                ),
                'alipaysecret' => array(
                    'title' => __('支付宝公钥', 'woocommerce'),
                    'type' => 'text',
                    'default' => '',
                    'description' => __('在支付宝开放平台上面创建应用的时候生成的支付宝公钥', 'woocommerce'),
                    'css' => 'width:400px'
                ),
                'pid' => array(
                    'title' => __('卖家支付宝PID', 'woocommerce'),
                    'type' => 'text',
                    'default' => '',
                    'description' => __('卖家支付宝用户号，以2088开头', 'woocommerce'),
                    'css' => 'width:400px'
                )            
            );
        }

        function process_payment($order_id) {
            global $woocommerce;
            global $aop;
            $file = fopen("pagepay.php", "w");
            
            $order = new WC_Order($order_id);
            $total_amount = $order->get_total();
            $order_title = $this->get_order_title($order);
            //wc_add_notice("errcode:500,errmsg:'错啦'",'error');
            // 调用支付宝支付接口

            $request = new AlipayTradePagePayRequest();
            $return_url = $this->get_return_url($order);
            $notify_url = home_url();
            $request->setReturnUrl($return_url); // 同步返回地址，http//https开头
            $request->setNotifyUrl($notify_url); // 支付宝服务器主动通知商户服务器里指定的页面http/https路径
            $request->setBizContent('{"product_code":"FAST_INSTANT_TRADE_PAY","out_trade_no": "' .$order_id . '", "subject": "' .$order_title.'", "total_amount": "'.$total_amount . '"}');   // 业务请求参数的集合，最大长度不限
            
            try{
                $response = $aop->pageExecute($request); //请求支付，返回的是一个表单
                // htmlspecialchars是为了输出到页面时防止被浏览器将关键参数html转义
                // var_dump($response); // 直接输出表单没用，需要将表单保存成一个文件，然后重定向到这个文件

                fwrite($file, $response);
                fclose($file);
                //wc_add_notice("errcode:500,errmsg:'错啦'",'error');
                
                // 返回感谢页面重定向
                return array(
                    'result' => 'success',
                    'redirect' => home_url() . '/pagepay.php'
                );
            }
            catch (Exception $e) {
                echo "hhhhh";
                wc_add_notice("errcode:{$e->getCode()},errmsg:{$e->getMessage()}",'error');
                return array(
                    'result' => 'fail',
                    'redirect' => $this->get_return_url($order)
                );
            }
        }

        private function get_order_title($order) {
            $order_id = $order->get_id();
            $title = "#{$order_id}";
            $order_items = $order->get_items();  // 获取这个订单中包含的商品
            if($order_items) {
                foreach($order_items as $item_id=>$item) {
                    $title.="|{$item['name']}";
                }
            }
            $limit = 250;
            $title = mb_strimwidth($title, 0, $limit, "utf-8");
            return $title;

        }

    }
    global $wc_getway_alipay;
    $wc_getway_alipay = new WC_Gateway_Alipay();
}
// 告诉WC这个类的存在
function add_my_gateway_class($methods) {
    $methods[] = 'WC_Gateway_Alipay';
    return $methods;
}
add_filter('woocommerce_payment_gateways', 'add_my_gateway_class');

/** 
 * 首先使用支付宝公钥检验是否是支付宝发送的消息；
 * 验签成功后对支付结果中的业务内容进行二次校验：
 *      out_trade_no是否为商户系统中创建的订单号；
 *      total_amount是否为该订单的实际金额；  
 *      seller_id（或者seller_email) 是否为out_trade_no这笔单据的对应的操作方（有的时候，一个商户可能有多个seller_id/seller_email）；
 *      app_id是否为商户本身；
 */
function check_notify($arr) {
    $arr['fund_bill_list'] = stripslashes($arr['fund_bill_list']);
    $aop = new AopClient();
    global $wc_getway_alipay;
    $aop->alipayrsaPublicKey = $wc_getway_alipay->alipaysecret;
    $flag = $aop->rsaCheckV1($arr, $wc_getway_alipay->alipaysecret, $wc_getway_alipay->signtype);

    if($flag){
        fwrite($file, "验签验证通过");

        // 商户订单号
        $out_trade_no = $arr['out_trade_no'];
        $order = new WC_Order($out_trade_no);
        // 不是商户系统中的订单号
        if(!$order){
            fclose($file);  
            return false;
        }
        // 订单金额
        $total_amount = $arr['total_amount'];
        // 订单金额有问题
        if($total_amount != $order->get_total()){
            return false;
        }
        // 卖家支付宝用户号，以2088开头
        $seller_id = $arr['seller_id'];
        if($seller_id != $wc_getway_alipay->pid){
            return false;
        }
        // APPID
        $app_id = $arr['app_id'];
        if($app_id != $wc_getway_alipay->appid){
            return false;
        }    

        return true;
          
    }else{
        return $flag;
    }  
}
add_action('init', 'notify');
/**
 * 处理来自支付宝服务器发送的异步通知，支付是否成功应该以异步通知为准。
 * 如果确定订单支付没什么问题，应该返回success信息，否则支付宝会在24小时内按一定时间策略重发通知。
 * 所有验证都成功后，继续商户自身的业务处理。
 */
function notify() {
    // 确定是来自支付宝的异步通知
    if(isset($_POST['notify_type']) && $_POST['notify_type']=='trade_status_sync'){
        global $wc_getway_alipay;
        $arr = $_POST;
        // 验证
        $flag = check_notify($arr);
        if($flag) {
            // 交易状态
            $trade_status = $_POST['trade_status'];
            // 在产品不支持退款的前提下，买家付款成功，或者支持退款的前提下，交易成功且超过可退款期限，这时会触发交易状态为TRADE_FINISHED的通知
            // 产品支持退款的情况下，买家付款成功会触发交易状态为TRADE_SUCCESS的通知
            if($trade_status == 'TRADE_FINISHED' || $trade_status=='TRADE_SUCCESS') {
                // 判断该订单是否在商户网站中已经做过处理，如果没有的话执行商户的业务程序
                $order = new WC_Order($_POST['out_trade_no']);
                if($order->needs_payment()){
                    $order->payment_complete();
                }   
            } 
            echo 'success';   
        }else {
            echo 'fail';
        }        
    }
}

?>